@@ -1,11 +1,12 @@
-#! perl
+#! perl --			-*- coding: utf-8 -*-
+
+use utf8;
 
-# RCS Id          : $Id: DB.pm,v 1.59 2008/04/14 14:39:20 jv Exp $
 # Author          : Johan Vromans
 # Created On      : Sat May  7 09:18:15 2005
 # Last Modified By: Johan Vromans
-# Last Modified On: Sun Apr 13 18:06:04 2008
-# Update Count    : 431
+# Last Modified On: Sun Jan 29 20:22:37 2017
+# Update Count    : 449
 # Status          : Unknown, Use with caution!
 
 ################ Common stuff ################
@@ -19,10 +20,9 @@ package EB::DB;
 use strict;
 use warnings;
 
-our $VERSION = sprintf "%d.%03d", q$Revision: 1.59 $ =~ /(\d+)/g;
-
 use EB;
 use DBI;
+use File::Glob ( $] >= 5.014 ? ":bsd_glob" : ":glob" );
 
 my $dbh;			# singleton for DB
 
@@ -50,7 +50,7 @@ sub check_db {
     warn(join(" ", sort keys %tables)."\n") if $fail;
     die("?".__x("Ongeldige EekBoek database: {db}.",
 		db => $dbh->{Name}) . " " .
-	_T("Wellicht is de database nog niet genitialiseerd?")."\n") if $fail;
+	_T("Wellicht is de database nog niet geïnitialiseerd?")."\n") if $fail;
 
     # Check version, and try automatic upgrade.
     my ($maj, $min, $rev)
@@ -60,11 +60,11 @@ sub check_db {
 	      sprintf("%03d%03d", $min, $rev) eq sprintf("%03d%03d", SCM_MINVERSION, SCM_REVISION)) ) {
 	# Basically, this will migrate to the highest possibly version, and then retry.
 	my $cur = sprintf("%03d%03d%03d", $maj, $min, $rev);
-	my $tmpl = EB_LIB . "EB/migrate/$cur?????????.*l";
+	my $tmpl = libfile("migrate/$cur?????????.*l");
 	my @a = reverse sort glob($tmpl);
 	last unless @a == 1;
 
-	if ( $a[0] =~ /\.sql$/ && open(my $fh, "<$a[0]")) {
+	if ( $a[0] =~ /\.sql$/ && open(my $fh, "<:encoding(utf-8)", $a[0])) {
 	    warn("!"._T("De database wordt aangepast aan de nieuwere versie")."\n");
 
 	    local($/);		# slurp mode
@@ -159,6 +159,11 @@ sub setup {
     setupdb();
 
     # Create temp table for account mangling.
+    # This table has the purpose of copying the data from Accounts, so that
+    # data from already completed financial years can be corrected when
+    # creating overviews, such as Balance statements and Result accounts.
+    # This way no backdated calculations need to be made when transitions
+    # to previous financial years are involved.
     my $sql = "SELECT * INTO TEMP TAccounts FROM Accounts WHERE acc_id = 0";
     $sql = $self->feature("filter")->($sql) if $self->feature("filter");
     $dbh->do($sql) if $sql;
@@ -185,7 +190,8 @@ sub store_journal {
     my ($self, $jnl) = @_;
     foreach ( @$jnl ) {
 	$self->sql_insert("Journal",
-			  [qw(jnl_date jnl_dbk_id jnl_bsk_id jnl_bsr_date jnl_bsr_seq jnl_acc_id jnl_amount
+			  [qw(jnl_date jnl_dbk_id jnl_bsk_id jnl_bsr_date jnl_bsr_seq jnl_seq
+			      jnl_type jnl_acc_id jnl_amount
 			      jnl_damount jnl_desc jnl_rel jnl_rel_dbk  jnl_bsk_ref)],
 			  @$_);
     }
@@ -234,7 +240,7 @@ sub bskid {
     if ( $nr =~ /^([[:alpha:]][^:]+)(?::([^:]+))?:(\d+)$/ ) {
 	$rr = $self->do("SELECT dbk_id, dbk_desc".
 			" FROM Dagboeken".
-			" WHERE dbk_desc ILIKE ?", $1);
+			" WHERE upper(dbk_desc) LIKE ?", uc($1));
 	unless ( $rr ) {
 	    return wantarray ? (undef, undef, __x("Onbekend dagboek: {dbk}", dbk => $1)) : undef;
 	}
@@ -570,7 +576,14 @@ sub set_sequence {
 
 sub _loaddbbackend {
     my ($self) = @_;
-    my $dbtype = $cfg->val(qw(database driver), "postgres");
+    my $dbtype = $cfg->val(qw(database driver), "sqlite");
+
+    # Trim whitespace for stupid users.
+    for ( $dbtype ) {
+	s/^\s+//;
+	s/\s+$//;
+    }
+
     my $pkg = __PACKAGE__ . "::" . ucfirst(lc($dbtype));
     my $pkgfile = __PACKAGE__ . "::" . ucfirst(lc($dbtype)) . ".pm";
     $pkgfile =~ s/::/\//g;
@@ -682,6 +695,20 @@ sub do {
     $rr;
 }
 
+sub da {			# do_all
+    my $self = shift;
+    my $sql = shift;
+    my $atts = ref($_[0]) eq 'HASH' ? shift : undef;
+    my @args = @_;
+    my $sth = $self->sql_exec($sql, @args);
+    my $res;
+    while ( my $rr = $sth->fetchrow_arrayref ) {
+	push( @$res, [@$rr] );
+    }
+    $sth->finish;
+    $res;
+}
+
 sub errstr {
     $dbh->errstr;
 }
@@ -701,14 +728,8 @@ sub checktx {
     $allow ? warn("!$msg\n") : die("?$msg\n");
 }
 
-sub rollback {
-    my ($self) = @_;
-    warn("=> ROLLBACK WORK;", $dbh ? "" : " (ignored)", "\n") if $trace;
-    return unless $dbh;
-    die("?INTERNAL ERROR: ROLLBACK while not in transaction\n") unless $tx;
-    $tx = 0;
-    $dbh->rollback
-}
+#
+# http://en.wikipedia.org/wiki/Database_transaction#In_SQL 
 
 sub begin_work {
     my ($self) = @_;
@@ -727,6 +748,15 @@ sub commit {
     $dbh->commit;
 }
 
+sub rollback {
+    my ($self) = @_;
+    warn("=> ROLLBACK WORK;", $dbh ? "" : " (ignored)", "\n") if $trace;
+    return unless $dbh;
+    die("?INTERNAL ERROR: ROLLBACK while not in transaction\n") unless $tx;
+    $tx = 0;
+    $dbh->rollback
+}
+
 END {
     prepstats();
     disconnectdb();
